#include "Man.h"
#include "Map.h"
#include "ScanKeys.h"
// #include "Chars.h"
#include "Vram.h"
#include "VVram.h"
#include "Main.h"
#include "Knife.h"
#include "Sound.h"

static const byte[] Chars = {
    // left
    0x80, 0x20, 0xa3, 0xaa,
    0x1f, 0xaa, 0xa3, 0xa9,
	0x80, 0x20, 0x83, 0xaa,
    // right
    0x20, 0x1f, 0xa9, 0xa2,
	0xa9, 0x1f, 0xaa, 0xa2,
    0x20, 0x1f, 0xa9, 0x82,
    // up
	0x80, 0xa3, 0x20, 0xab,
    0x80, 0xa3, 0xab, 0xa9,
	0x80, 0x91, 0x20, 0xab,
    // down
    0x20, 0xa9, 0x80, 0xa2,
	0xa9, 0xab, 0x80, 0xa2,
    0x20, 0xa9, 0x80, 0x90,
};
struct Direction {
    byte key;
    sbyte dx, dy;
    ptr<byte> pChars;
    byte index;
    sbyte xOffset, yOffset;
};
static const Direction[] Directions = {
    { Keys_Left, -1, 0, Chars + 4 * 3 * 0, 0, 0, 0 },
    { Keys_Right, 1, 0, Chars + 4 * 3 * 1, 1, 1, 0 },
    { Keys_Up, 0, -1, Chars + 4 * 3 * 2, 2, 0, 0 },
    { Keys_Down, 0, 1, Chars + 4 * 3 * 3, 3, 0, 1 },
};

byte ManX, ManY;
bool ManLost;
static ptr<Direction> pPrevDirection;
static ptr<byte> pChars;
static byte seqIndex;
static bool buttonOn;

static void UpdateBasePosition()
{
    if (ManX < VVramWidth / 2) {
        BaseX = 0;
    }
    else {
        byte x, margin;
        margin = MapWidth - VVramWidth;
        x = ManX - VVramWidth / 2;
        if (x >= margin) {
            BaseX = margin;
        }
        else {
            BaseX = x;
        }
    }
    if (ManY < VVramHeight / 2) {
        BaseY = 0;
    }
    else {
        byte y, margin;
        margin = MapHeight - VVramHeight;
        y = ManY - VVramHeight / 2;
        if (y >= margin) {
            BaseY = margin;
        }
        else {
            BaseY = y;
        }
    }
}

void InitMan()
{
    ManX = ToMapSize(*pStage); ++pStage;
    ManY = ToMapSize(*pStage); ++pStage;
    UpdateBasePosition();
    pPrevDirection = Directions + 1; // right
    pChars = pPrevDirection->pChars;
    seqIndex = 0;
}


void DrawMan()
{
    VPut2(ManX, ManY, pChars);
}


void MoveMan()
{
    byte key;
    ptr<Direction> pDirection;
    byte x, y;
    if (ManLost) return;
    
    key = ScanKeys();
    for (pDirection : Directions) {
        if ((key & pDirection->key) != 0) {
            x = ManX + pDirection->dx;
            y = ManY + pDirection->dy;
            if (CanMove2(x, y)) {
                pPrevDirection = pDirection;
                goto canMove;
            }
        }
    }
    if ((key & Keys_Dir) != 0) {
        x = ManX + pPrevDirection->dx;
        y = ManY + pPrevDirection->dy;
        if (CanMove2(x, y)) {
            canMove:
            ManX = x;
            ManY = y;
            UpdateBasePosition();
            if (x == GoalX && y == GoalY) {
                StageClear = true;
            }
            seqIndex = (seqIndex + 1) & 1;
            // if (seqIndex == 3) {
            //     seqIndex = 1;
            // }
            pChars = pPrevDirection->pChars + ((1 + seqIndex) << 2);
            goto testButton;
        }
    }
    pChars = pPrevDirection->pChars;
    testButton:
    if ((key & Keys_Button0) != 0 && !buttonOn) {
        byte x, y;
        x = ManX + pPrevDirection->dx;
        y = ManY + pPrevDirection->dy;
        if (CanMove1(x, y)) {
            StartKnife(x, y, pPrevDirection->index);
            buttonOn = true;
            return;
        }
    }
    else {
        buttonOn = false;
    }
}


void EndMan()
{
    static const byte[] Chars = {
        0x80, 0x20, 0xa3, 0xaa,
        0x20, 0x1f, 0xa9, 0xa2,
        0xab, 0xa3, 0x20, 0x80, 
        0xa2, 0x80, 0xac, 0x20,
    };
    byte i;
    i = 0;
    repeat (10) {
        Sound_Loose();
        VPut2(ManX, ManY, Chars + i);
        DrawAll();
        i = (i + 4) & 0x0c;
    }
}